@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014 multiple authors, see README for licence details
+ * Copyright multiple authors, see README for licence details
  */
 #include <stdio.h>
 #include <signal.h>
@@ -16,14 +16,15 @@
 #include "fns.h"
 
 char *version[] = {
-	"9wm version 1.3.5, Copyright © 2015 multiple authors", 0,
+	"9wm version 1.3.9, Copyright (c) 2016 multiple authors", 0,
 };
 
 Display *dpy;
 ScreenInfo *screens;
 int initting;
 XFontStruct *font;
-int nostalgia;
+int curs;
+int border;
 char **myargv;
 char *termprog;
 char *shell;
@@ -44,6 +45,8 @@ Atom wm_delete;
 Atom wm_take_focus;
 Atom wm_colormaps;
 Atom wm_moveresize;
+Atom net_wm_state;
+Atom net_wm_state_fullscreen;
 Atom active_window;
 Atom utf8_string;
 Atom _9wm_running;
@@ -70,35 +73,72 @@ sigchld(int signum)
 void
 usage(void)
 {
-	fprintf(stderr, "usage: 9wm [-version] [-nostalgia] [-font fname] [-term prog] [exit|restart]\n");
+	fprintf(stderr, "usage: 9wm [-version] [-cursor cursor] [-border] [-font fname] [-term prog] [-active color] [-inactive color] [exit|restart]\n");
 	exit(1);
 }
 
+#ifdef COLOR
+char *activestr = NULL;
+char *inactivestr = NULL;
+
+unsigned long
+getcolor(Colormap cmap, char *str)
+{
+	if (str != NULL) {
+		XColor color;
+		Status stpc = 0;
+		if (cmap != 0)
+			stpc = XParseColor(dpy, cmap, str, &color);
+		Status stac = 0;
+		if (stpc != 0)
+			stac = XAllocColor(dpy, cmap, &color);
+		if (stac != 0) {
+			return color.pixel;
+		}
+	}
+	return 0;
+}
+#endif
+
 int
 main(int argc, char *argv[])
 {
 	int i, do_exit, do_restart;
 	char *fname;
 	int shape_event, dummy;
-
 	myargv = argv;		/* for restart */
 
 	do_exit = do_restart = 0;
 	font = 0;
 	fname = 0;
 	for (i = 1; i < argc; i++)
-		if (strcmp(argv[i], "-nostalgia") == 0)
-			nostalgia++;
-		else if (strcmp(argv[i], "-debug") == 0)
+		if (strcmp(argv[i], "-debug") == 0)
 			debug++;
-		else if (strcmp(argv[i], "-font") == 0 && i + 1 < argc) {
-			i++;
-			fname = argv[i];
-		} else if (strcmp(argv[i], "-term") == 0 && i + 1 < argc)
+		else if (strcmp(argv[i], "-font") == 0 && i + 1 < argc)
+			fname = argv[++i];
+		else if (strcmp(argv[i], "-term") == 0 && i + 1 < argc)
 			termprog = argv[++i];
 		else if (strcmp(argv[i], "-version") == 0) {
 			fprintf(stderr, "%s\n", version[0]);
 			exit(0);
+		} else if (strcmp(argv[i],"-border") == 0)
+			border++;
+		else if (strcmp(argv[i],"-cursor")  == 0 && i + 1 < argc) {
+			i++;
+			if (strcmp(argv[i],"v1") == 0)
+				curs = 1;
+			else if (strcmp(argv[i],"blit") == 0)
+				curs = 2;
+		} else if ( (strcmp(argv[i], "-active") == 0 || strcmp(argv[i], "-inactive") == 0) && i + 1 < argc) {
+#ifdef COLOR
+			if(argv[i][1] == 'a')
+				activestr = argv[++i];
+			else
+				inactivestr = argv[++i];
+#else
+			fprintf(stderr,"9wm: border set but COLOR not defined\n");
+			i++;
+#endif
 		} else if (argv[i][0] == '-')
 			usage();
 		else
@@ -155,6 +195,8 @@ main(int argc, char *argv[])
 	wm_colormaps = XInternAtom(dpy, "WM_COLORMAP_WINDOWS", False);
 	wm_moveresize = XInternAtom(dpy, "_NET_WM_MOVERESIZE", False);
 	active_window = XInternAtom(dpy, "_NET_ACTIVE_WINDOW", False);
+	net_wm_state = XInternAtom(dpy, "_NET_WM_STATE",False);
+	net_wm_state_fullscreen = XInternAtom(dpy, "_NET_WM_STATE_FULLSCREEN", False);
 	utf8_string = XInternAtom(dpy, "UTF8_STRING", False);
 	_9wm_running = XInternAtom(dpy, "_9WM_RUNNING", False);
 	_9wm_hold_mode = XInternAtom(dpy, "_9WM_HOLD_MODE", False);
@@ -176,7 +218,7 @@ main(int argc, char *argv[])
 				break;
 		}
 	}
-	if (nostalgia == BLIT) {
+	if (border) {
 		_border--;
 		_inset--;
 	}
@@ -237,7 +279,24 @@ initscreen(ScreenInfo * s, int i)
 
 	s->black = BlackPixel(dpy, i);
 	s->white = WhitePixel(dpy, i);
-
+	/*
+	 * Setup color for border
+	 */
+	s->active = s->black;
+	s->inactive = s->white;
+#ifdef COLOR
+	if (activestr != NULL || inactivestr != NULL) {
+		Colormap cmap = DefaultColormap(dpy,s->num);
+		if (cmap != 0) {
+			unsigned long active = getcolor(cmap,activestr);
+			if (active != 0)
+				s->active = active;
+			unsigned long inactive = getcolor(cmap,inactivestr);
+			if (inactive != 0)
+				s->inactive = inactive;
+		}
+	}
+#endif
 	gv.foreground = s->black ^ s->white;
 	gv.background = s->white;
 	gv.function = GXxor;
